#!/usr/bin/python
## -*- coding: utf-8 -*-

from wcs.services.mgrbase import MgrBase
from wcs.commons.config import Config
from wcs.commons.http import _post
from wcs.commons.http import _get
from wcs.commons.util import urlsafe_base64_encode,https_check,entry
from wcs.commons.logme import debug, error

MGR_URL = Config.mgr_url

class PersistentFop(MgrBase):
    """音视频持久化操作类
    该类继承自mgrbase
    """
    def __init__(self,auth,url):
        super(PersistentFop, self).__init__(auth,url)
      
    def _build_op(self,cmd, first_arg, **kwargs):
        op = [cmd]
        if first_arg is not None:
            op.append(first_arg)

        for k, v in kwargs.items():
            op.append('{0}/{1}'.format(k, v))
        return '/'.join(op)

    def _pipe_cmd(self,*cmds):
        return '|'.join(cmds)

    def _op_save(self,op, bucket, key):
        return self._pipe_cmd(op, 'saveas/' + entry(bucket, key))

    def build_ops(self,ops,bucket,key):
        ops_list = []
        for op,params in ops.items(): 
            ops_list.append(self._op_save(self._build_op(op,params),bucket,key))
        return ops_list

    def fops_status(self, persistentId):
        url = https_check('{0}/status/get/prefop?persistentId={1}'.format(self.mgr_host, persistentId))
        debug('Start to get status of persistentId: %s' % persistentId)
        return _get(url=url)

    def _gernerate_headers(self,url,data):
        reqdata = super(PersistentFop, self)._params_parse(data)
        headers = super(PersistentFop, self)._gernerate_headers(url, body=reqdata) 
        return headers,reqdata

    def execute(self,fops,bucket,key,force=0,separate=0,notifyurl=None):
        data = {'bucket': urlsafe_base64_encode(bucket), 'key': urlsafe_base64_encode(key), 'fops': urlsafe_base64_encode(fops)}
        if notifyurl is not None:
            data['notifyURL'] = urlsafe_base64_encode(notifyurl)
        if force == 1:
            data['force'] = 1
        if separate == 1:
            data['separate'] = 1
        url = https_check('{0}/fops'.format(self.mgr_host))
        headers,reqdata = self._gernerate_headers(url,data)
        debug('PersistentFops is %s' % fops)
        debug('Start to post persistentFops')
        return _post(url=url, data=reqdata,headers=headers)





#!/usr/bin/python
## -*- coding: utf-8 -*-

class MgrBase(object):
    """资源管理基类
    资源管理类接口有很多相似的部分，为了降低代码冗余，将公共部分写入基类，需要时继承基类即可
    Attributes:
        auth: 上传&管理token计算实例
        mgr_host: 管理域名
    """
    def __init__(self, auth,mgr_url):
        self.auth = auth
        self.mgr_host = mgr_url

    def _gernerate_headers(self,url,body=None):
        token = self.auth.managertoken(url,body=body)
        headers = {'Authorization': token}
        return headers
        
    def _params_parse(self, params):
        paramlist = []
        if params:
            for k, v in params.items():
                paramlist.append('{0}={1}'.format(k, v))
            paramlist = '&'.join(paramlist) 
        return paramlist




#!/usr/bin/python
## -*- coding: utf-8 -*-

import base64
import json
import os,sys
import tempfile
import shutil
from wcs.commons.config import Config

tmp_record_folder = Config.tmp_record_folder

class UploadProgressRecorder(object):
    """持久化上传记录类
    该类默认保存每个文件的上传记录到文件系统中，用于断点续传
    当前上传记录的格式是在tmp_record_folder目录下，生成已当前上传任务的upload id命名的目录，然后在目录tmp_record_folder/upload id下生成多个文件，每个文件以块offset命名，并记录了这个块的上传结果，上传结果为json格式：
    {
        "size": file size,
        "offset": position of block,
        "ctx": contexts
    }
    Attributes:
        record_folder: 保存上传记录的目录
    """
    def __init__(self,upload_id):
        tmp = tmp_record_folder + '/%s/' % upload_id
        if tmp is not None:
            if os.path.exists(tmp) is False:
                try:
                    os.makedirs(tmp)
                except Exception as e:
                    raise Exception('mkdir fail')
                    sys.exit()
            self.record_folder = tmp
            self.upload_id = upload_id
        else:
            raise Exception("tmp_record_folder is necessary!")

    def get_upload_record(self):
        results = {'uploadBatch':self.upload_id,'upload_record':[]}
        offsets = os.listdir(self.record_folder)
        if len(offsets) == 0:
            raise Exception('upload_records is Null, please redefine uploadBatch')
            sys.exit()
        for offset in offsets:
            path = os.path.join(self.record_folder, offset)
            try:
                with open(path, 'r') as f:
                    results['upload_record'].append(f.read())
            except Exception as e:
                raise Exception("Read upload progress fail")
                sys.exit()
        return results

    def set_upload_record(self, offset, data):
        upload_record_file_path = os.path.join(self.record_folder, str(offset))
        try:
            with open(upload_record_file_path, 'w') as f:
                json.dump(data, f)
        except Exception as e:
            raise Exception('Write upload progress fail')
            sys.exit()

    def delete_upload_record(self):
        try:
            shutil.rmtree(self.record_folder)
        except Exception as e:
            raise Exception('Delete upload progress fail')
            sys.exit()

    def find_upload_record(self):
        return os.path.exists(self.record_folder)




# -*- coding: utf-8 -*-

import base64
import hmac
try:
    import simplejson as json
except (ImportError, SyntaxError):
    import json
from wcs.commons.compat import urlparse
from hashlib import sha1
from wcs.commons.util import urlsafe_base64_encode, urlsafe_base64_decode
from datetime import datetime
import time

class Auth(object):

    """
    wcs auth 
    
    calculate upload token & manager token

    """

    def __init__(self, access_key, secret_key):
        
        self.checkKey(access_key, secret_key)
        self.access_key = access_key
        self.secret_key = secret_key

    def uploadtoken(self, putPolicy):
        """
        input: Put Policy
        return: uploadtoken
        """
        current = int(time.mktime(datetime.now().timetuple())) * 1000
        if 'deadline' not in putPolicy.keys() or putPolicy['deadline'] > current:
            putPolicy['deadline'] = current + 7200000

        jsonputPolicy = json.dumps(putPolicy)
        #encodePutPolicy = base64.b64encode(jsonputPolicy)
        encodePutPolicy = urlsafe_base64_encode(jsonputPolicy)
        Sign = hmac.new(self.secret_key.encode('utf-8'), encodePutPolicy.encode('utf-8'), sha1)    
        #encodeSign = base64.b64encode(Sign.hexdigest())
        encodeSign = urlsafe_base64_encode(Sign.hexdigest())
        return '{0}:{1}:{2}'.format(self.access_key, encodeSign, encodePutPolicy)

    def managertoken(self, url, body=None):
        """
        input: url & body(optional)
        return: managertoken
        """
        parsed_url = urlparse(url)
        query = parsed_url.query
        path = parsed_url.path
        if query:
            if body:
                signingStr = ''.join([path,'?',query,"\n",body]) 
            else:
                signingStr = ''.join([path,'?',query,"\n"])
        else:
            if body:
                signingStr = ''.join([path,"\n",body])
            else:
                signingStr = ''.join([path,"\n"])
        SignStr = hmac.new(self.secret_key.encode('utf-8'), signingStr.encode('utf-8'), sha1)    
        encodeSignStr = urlsafe_base64_encode(SignStr.hexdigest())
        return '{0}:{1}'.format(self.access_key,encodeSignStr)
    
    @staticmethod
    def checkKey(access_key,secret_key):
        if not (access_key and secret_key):
            raise ValueError('invalid key')


        













import argparse
from wcs.commons.util import file_crc64



def main():
    parser = argparse.ArgumentParser(prog='WCS Python SDK')
    sub_parsers = parser.add_subparsers()
    parser_etag = sub_parsers.add_parser(
        'crc64', description='calculating 64-bit CRC value', help='crc64 [file...]')
    parser_etag.add_argument(
        'crc64_files', metavar='N', nargs='+', help='tthe file list for calculate crc64')

    args = parser.parse_args()
    try:
        crc64_files = args.crc64_files
    except AttributeError:
        crc64_files = None

    if crc64_files:
        print(crc64_files)
        r = [file_crc64(file) for file in crc64_files]
        if len(r) == 1:
            print(r[0])
        else:
            print(' '.join([str(i) for i in r]))

if __name__ == '__main__':
    main()




import os
import logging
import errno
import time


#if os.path.exists(path) is False:
#    try:
#        os.mkdir(path)
#    except OSError as exc:
#        raise exc

logger = logging.getLogger("wcs_python_sdk")
time_obj = time.gmtime()
log_time = "{year}_{month}_{day}".format(year=time_obj.tm_year,month=time_obj.tm_mon,day=time_obj.tm_mday)

def debug(msg):
    logger.setLevel(logging.DEBUG)
    debughandler = logging.StreamHandler()
    debughandler.setLevel(logging.DEBUG)
    fmt = '%(asctime)s - %(levelname)s - %(message)s'
    formatter = logging.Formatter(fmt)
    debughandler.setFormatter(formatter)
    logger.addHandler(debughandler)
    logger.debug(msg)
    logger.removeHandler(debughandler)

def warning(msg):
    logger.setLevel(logging.WARNING)
    warnhandler = logging.StreamHandler()
    warnhandler.setLevel(logging.WARNING)
    fmt = '%(asctime)s - %(levelname)s - %(message)s' 
    formatter = logging.Formatter(fmt)
    warnhandler.setFormatter(formatter)
    logger.addHandler(warnhandler)
    logger.warning(msg)
    logger.removeHandler(warnhandler)

def error(msg):
    logger.setLevel(logging.ERROR)
    errorhandler = logging.StreamHandler()
    errorhandler.setLevel(logging.ERROR)
    fmt = '%(asctime)s - %(levelname)s - %(message)s' 
    formatter = logging.Formatter(fmt)
    errorhandler.setFormatter(formatter)
    logger.addHandler(errorhandler)
    logger.error(msg)
    logger.removeHandler(errorhandler)

        




#!/usr/bin/python
## -*- coding: utf-8 -*-

import os
from urllib.request import quote
import requests
from requests_toolbelt import MultipartEncoder
from wcs.commons.http import _post
from os.path import expanduser
from wcs.commons.logme import debug, error
from wcs.commons.util import https_check
from requests.adapters import HTTPAdapter
from wcs.commons.config import Config

#config_file = os.path.join(expanduser("~"), ".wcscfg")


class SimpleUpload(object):
    """普通上传类
    该类实现了WCS的普通上传功能
    Attributes:
        url: 上传域名    
    """

    def __init__(self, url):
        self.url = url
        session = requests.Session()
        session.mount('http://', HTTPAdapter(max_retries=int(Config.connection_retries)))
        session.mount('https://', HTTPAdapter(max_retries=int(Config.connection_retries)))
        global _session
        _session = session
        #self.cfg = Config(config_file)
        self.cfg = Config()

    def _gernerate_tool(self, f, token ,key):
        fileds = {"token": token}
        url = "{0}/{1}/{2}".format(self.url, "file", "upload")
        fileds['file'] = (quote(key), f, 'text/plain')
        encoder = MultipartEncoder(fileds)
        headers = {"Content-Type": encoder.content_type}
        headers['Expect'] = '100-continue'
        headers['user-agent'] = "WCS-Python-SDK-4.0.0(http://wcs.chinanetcenter.com)"
        m = encoder.to_string()
        return url, m, headers

    def _gernerate_content(self, path):
        return open(path, 'rb')

    def _upload(self, url, encoder, headers, f):
        url = https_check(url)
        if self.cfg.keepalive == True:
            pass
        else:
            _session.keep_alive = False
            headers['Connection'] = 'close'
        try:
            if self.cfg.isverify:
                if self.cfg.returnUrl:
                    r = _session.post(url=url, headers=headers, data=encoder, verify=True, allow_redirects=False)
                else:
                    r = _session.post(url=url, headers=headers, data=encoder, verify=True)
            else:
                if self.cfg.returnUrl:
                    r = _session.post(url=url, headers=headers, data=encoder, verify=False, allow_redirects=False)
                else:
                    r = _session.post(url=url, headers=headers, data=encoder, verify=False)
        except requests.ConnectionError as conn_error:
            debug('Url connection abnormal,please check url!')
            return -1,conn_error
        except Exception as e:
            f.close()
            debug('Request url:' + url)
            debug('Headers:')
            debug(headers)
            debug('Exception:')
            debug(e)
            return -1, e
        f.close()
        try:
            r_header = {'x-reqid': r.headers['x-reqid']}
            if r.status_code == 303:
                return r.status_code, r.headers['Location'], r_header
            else:
                return r.status_code, r.text, r_header
        except:
            return r.status_code, r.text

    def upload(self, filepath, token ,key='filename'):
        if os.path.exists(filepath) and os.path.isfile(filepath):
            f = self._gernerate_content(filepath)
            url, encoder, headers = self._gernerate_tool(f, token ,key)
            return self._upload(url, encoder, headers, f)
        else:
            error('Sorry ! Please input a existing file')
            raise ValueError("Sorry ! We need a existing file to upload")



#!/usr/bin/python 
#coding=utf-8
#@TIME : 2018/3/5 11:06
#@AUTHOR : CAIYZ
#@FILE : error_deal
CLIENT_ERROR_STATUS = -1
WCS_REQUEST_ID = 'X-Reqid'
class ParameterError(Exception):
    '''
    自定义参数错误异常
    Parameter  error
    '''
    def __init__(self,parameter):
        super(ParameterError,self).__init__()
        self.parameter = parameter

    def __str__(self):
        return self.parameter
class WcsSeriveError(Exception):
    '''
    server error
    '''
    def __init__(self,seriveerror):
        super(WcsSeriveError,self).__init__()
        self.seriveerror = seriveerror

    def __str__(self):
        return self.seriveerror


class WcsError(Exception):
    def __init__(self, status, headers, body, details):
        #: HTTP 状态码
        self.status = status

        #: 请求ID，用于跟踪一个请求。
        self.request_id = headers.get(WCS_REQUEST_ID, '')

        #: HTTP响应体（部分）
        self.body = body

        #: 详细错误信息，是一个string到string的dict
        self.details = details

        #: 错误码
        self.code = self.details.get('Code', '')

        #: 错误信息
        self.message = self.details.get('Message', '')

    def __str__(self):
        error = {'status': self.status,
                 WCS_REQUEST_ID : self.request_id,
                 'details': self.details}
        return str(error)

    def _str_with_body(self):
        error = {'status': self.status,
                 WCS_REQUEST_ID : self.request_id,
                 'details': self.body}
        return str(error)

class ClientError(WcsError):
    def __init__(self, message):
        WcsError.__init__(self,CLIENT_ERROR_STATUS, {}, 'ClientError: ' + message, {})

    def _str_with_body(self):
        error = {'status': self.status,
                 WCS_REQUEST_ID : self.request_id,
                 'details': self.body}
        return str(error)

    def __str__(self):
        return self._str_with_body()




# -*- coding: utf-8 -*-
import os
import re
import sys
import io
from inspect import isfunction

from wcs.commons.logme import debug,error,warning

class Config(object):

    _instance = None
    _parsed_files = []
    access_key = ''
    secret_key = ''
    put_url = ''
    mgr_url = ''
    # 设置分片上传的块大小和片大小，单位均为字节
    block_size = 1024 * 1024 * 4
    bput_size = 512 * 1024

    # 设置请求连接超时重传次数
    connection_retries = 3
    connection_timeout = 40

    # 设置重传失败重试次数
    mkblk_retries = 3
    bput_retries = 3
    mkfile_retries = 3

    # 记录分片上传进度
    tmp_record_folder = '/tmp/multipart/'

    # 设置分片上传并发线程数
    # 如果留空，默认为机器的CPU核数，但是考虑带宽问题，一般设为<=计算机核数
    concurrency = 4
    # 是否支持https
    ishttps = False
    #是否开启证书校验
    isverify = True
    #keep-alive 配置
    keepalive = False

    # 是否断点续传
    upload_id = ''
 
    #可选参数output
    output = ''
    
    #可选参数notifyURL
    notifyurl = ''
    
    #可选参数separate
    separate = 0

    #可选参数force
    force = 0

    #bucket list可选参数limit
    limit = 1000
    prefix = ''
    mode = 3
    marker = ''

    #上传策略参数
    overwrite = 0
    returnUrl = ''
    returnBody = ''
    callbackUrl = ''
    callbackBody = ''
    persistentNotifyUrl = ''
    persistentOps = ''
    contentDetect = ''
    detectNotifyURL = ''
    detectNotifyRule = ''
    hashAlgorithm = ''

    ## Creating a singleton
    def __new__(self, configfile=None, ak=None, sk=None, put_url=None, mgr_url=None ):
        if self._instance == None:
            self._instance = object.__new__(self)
        return self._instance

    def __init__(self, configfile=None):
        if configfile:
            try:
                self.read_config_file(configfile)
            except IOError as e:  # 这里尚未确定读取配置文件异常如何处理
                error("Can't read config file ")
                sys.exit()

    def option_list(self):
        retval = []
        #funclist = ['option_list', 'read_config_file','dump_config','update_option']
        funclist = [x for x in Config.__dict__.keys() if isfunction(Config.__dict__[x])] #caiy 20180305,动态获取类的方法列表
        for option in dir(self):
            if option.startswith("_") or option in funclist:
                continue
            retval.append(option)
        return retval

    def read_config_file(self, configfile):
        cp = ConfigParser(configfile)
        for option in self.option_list():
            _option = cp.get(option)
            if _option is not None:
                _option = _option.strip()
            self.update_option(option, _option)
        self._parsed_files.append(configfile)

    def dump_config(self, stream):
        ConfigDumper(stream).dump(u"default", self)

    def update_option(self, option, value):
        if value is None:
            return
        if type(getattr(Config, option)) is type(True):
            if str(value).lower() in ("true","yes","on","1"):
                value = True
            elif str(value).lower() in ("false","no","off","0"):
                value = False
            else:
                raise ValueError("Config: value of option '%s' must be Yes or No, not '%s'" % (option, value))
        setattr(Config, option, value)

class ConfigParser(object):
    def __init__(self, file, sections = []):
        self.cfg = {}
        self.parse_file(file, sections)

    def parse_file(self, file, sections=[]):
        #debug("ConfigParse: Reading file '%s'" % file)
        if type(sections) != type([]):
            sections = [sections]
        in_our_section = True
        r_section = re.compile("^\[([^\]]+)\]")
        r_comment = re.compile("^\s*#.*")
        r_empty = re.compile("\s*$")
        r_data = re.compile("^\s*(?P<key>\w+)\s*=\s*(?P<value>.*)")
        r_quotes = re.compile("^\"(.*)\"\s*$") #匹配双引号
        with io.open(file, "r", encoding=self.get('encoding', 'UTF-8')) as fp:
            for line in fp:
                if r_comment.match(line) or r_empty.match(line):
                    continue
                is_section = r_section.match(line)
                if is_section:
                    section = is_section.group()[0]
                    in_our_section = (section in sections) or (len(sections) == 0)
                    continue
                is_data = r_data.match(line)
                if is_data and in_our_section:
                    data = is_data.groupdict()
                    if r_quotes.match(data["value"]):
                        data["value"] = data["value"][1:-1]
                    self.__setitem__(data["key"],data["value"])
                    if data["key"] in ("access_key","secret_key"):
                        try: #caiyz 20180315 添加异常处理
                            print_value = ("%s...%d_chars...%s") % (data["value"][-2], len(data["value"]) - 3, data["value"][-1:1])
                        except IndexError as e:
                            debug(u"{} is empty".format(data["key"]))
                    else:
                        print_value = data["value"]
                    #debug("ConfigParser: %s->%s" % (data["key"], print_value))
                    continue
                warning("Ingnoring invalid line in '%s': %s" % (file, line))
    def __geteitem__(self, name):
        return self.cfg[name]

    def __setitem__(self, name, value):
        self.cfg[name] = value

    def get(self, name, default = None):
        if name in self.cfg:
            return self.cfg[name]
        return default

class ConfigDumper(object):
    def __init__(self, stream):
        self.stream = stream

    def dump(self, section, config):
        self.stream.write(u"[%s]\n" % section)
        for option in config.option_list():
            value = getattr(config, option)
            self.stream.write(u"%s = %s\n" % (option, value))



#!/usr/bin/python
## -*- coding: utf-8 -*-
import os

from wcs.commons.util import urlsafe_base64_encode
from wcs.commons.auth import Auth
from wcs.services.simpleupload import SimpleUpload
from wcs.services.streamupload import StreamUpload
from wcs.services.multipartupload import MultipartUpload
from wcs.services.filemanager import BucketManager
from wcs.services.fmgr import Fmgr
from wcs.services.persistentfop import PersistentFop
from wcs.services.wslive import WsLive
from wcs.commons.putpolicy import PutPolicy
from wcs.commons.error_deal import WcsSeriveError
class Client(object):
    """接口封装类
    该类封装了SDK提供的全部API，用户在开发时只需要实例化这个类就可以调用SDK提供的全部接口，而不需要根据不同的API实例化不同的类
    Attributes:
        auth: 上传&管理token计算实例
        simpleupload: 普通上传实例
        streamupload: 流地址上传实例
        multiupload: 分片上传实例
        bmgr: 资源管理实例
        fmgr: 高级资源管理实例
        pfops: 音视频持久化操作实例
        wsl: 直播录制文件列举实例
        cfg: 配置文件管理实例
    """
    def __init__(self, config):
        self.auth = Auth(config.access_key, config.secret_key)
        self.simpleupload = SimpleUpload(config.put_url)
        self.streamupload = StreamUpload(config.put_url)
        self.multiupload = MultipartUpload(config.put_url)
        self.bmgr = BucketManager(self.auth,config.mgr_url)
        self.fmgr = Fmgr(self.auth,config.mgr_url)
        self.pfops = PersistentFop(self.auth,config.mgr_url)
        self.wsl = WsLive(self.auth,config.mgr_url)
        self.cfg = config

    def simple_upload(self, path, bucket, key):
        policy = PutPolicy()
        policy.set_conf('scope', '%s:%s' % (bucket,key))
        policy.dump_policy(self.cfg)
        token = self.auth.uploadtoken(policy.putpolicy)
        return self.simpleupload.upload(path,token ,key)

    def stream_upload(self, stream, bucket, key):
        policy = PutPolicy()
        policy.set_conf('scope', '%s:%s' % (bucket,key))
        policy.dump_policy(self.cfg)
        token = self.auth.uploadtoken(policy.putpolicy)
        return self.streamupload.upload(stream,token ,key)

    def multipart_upload(self,path,bucket, key,tmp_upload_id=None):
        policy = PutPolicy()
        policy.set_conf('scope', '%s:%s' % (bucket,key))
        policy.dump_policy(self.cfg)
        token = self.auth.uploadtoken(policy.putpolicy)
        upload_id = tmp_upload_id or self.cfg.upload_id
        return self.multiupload.upload(path,token,upload_id)

    def smart_upload(self,path,bucket, key,tmp_upload_id=None,multi_size=20):
        policy = PutPolicy()
        policy.set_conf('scope', '%s:%s' % (bucket,key))
        policy.dump_policy(self.cfg)
        token = self.auth.uploadtoken(policy.putpolicy)
        file_size = 1024*1024*int(multi_size)
        upload_id = tmp_upload_id or self.cfg.upload_id
        if os.path.getsize(path) <= file_size and not upload_id:
            upload_result = self.simpleupload.upload(path ,token ,key)
            if 200 <= upload_result[0] <400:
                return upload_result
            else:
                raise WcsSeriveError('Upload file fail,erorr info:{0}'.format(upload_result))
        else:
            return self.multiupload.smart_upload(path,token,upload_id)

    def bucket_list(self,bucket,prefix=None, marker=None, limit=None, mode=None, starttime=None,endtime=None):
        try:
            pre = prefix or str(self.cfg.prefix)
        except Exception:
            pre = ''
        
        try:
            # m = mode or int(self.cfg.mode)
            if mode == None:
                m = int(self.cfg.mode)
            else:
                m = int(mode)
        except Exception:
            m = ''
        
        try:
            mar = marker or str(self.cfg.marker)
        except Exception:
            mar = ''
        try:
            l = limit or int(self.cfg.limit)
        except Exception:
            l = ''
        return self.bmgr.bucketlist(bucket,pre,mar,l,m, starttime, endtime)

    def list_buckets(self):
        return self.bmgr.bucket_list()
   
    def bucket_stat(self, name, startdate, enddate):
        return self.bmgr.bucket_stat(name, startdate, enddate)
   
    def bucket_statistics(self, name, stype, startdate, enddate, isListDetails='false'):
        return self.bmgr.bucket_statistics(name, stype, startdate, enddate, isListDetails)
    
    def image_detect(self, image, dtype, bucket):
        return self.bmgr.image_detect(image, dtype, bucket)

    def stat(self,bucket,key):
        return self.bmgr.stat(bucket,key)

    def delete(self,bucket,key):
        return self.bmgr.delete(bucket,key)

    def move(self,srcbucket, srckey, dstbucket, dstkey=''):
        if dstkey:
            pass
        else:
            dstkey = srckey
        return self.bmgr.move(srcbucket, srckey, dstbucket, dstkey)

    def copy(self,srcbucket, srckey, dstbucket, dstkey=''):
        if dstkey:
            pass
        else:
            dstkey = srckey
        return self.bmgr.copy(srcbucket, srckey, dstbucket, dstkey)

    def setdeadline(self,bucket,key,deadline):
        return self.bmgr.setdeadline(bucket,key,deadline)

    def _parse_fops(self, fops):
        data = [fops]
        if self.cfg.notifyurl:
            data.append('notifyURL=%s' % urlsafe_base64_encode(self.cfg.notifyurl))
        if self.cfg.separate: 
            data.append('separate=%s' % self.cfg.separate)
        if self.cfg.force:
            data.append('force=%s' % self.cfg.force)
        return 'fops=' + '&'.join(data)

    def fmgr_move(self, fops):
        return self.fmgr.fmgr_move(self._parse_fops(fops))

    def fmgr_copy(self, fops): 
        return self.fmgr.fmgr_copy(self._parse_fops(fops))

    def fmgr_fetch(self, fops):
        return self.fmgr.fmgr_fetch(self._parse_fops(fops))

    def fmgr_delete(self, fops):
        return self.fmgr.fmgr_delete(self._parse_fops(fops))

    def prefix_delete(self, fops):
        return self.fmgr.prefix_delete(self._parse_fops(fops))

    def m3u8_delete(self, fops):
        return self.fmgr.m3u8_delete(self._parse_fops(fops))

    def fmgr_compress(self,fops):
        return self.fmgr.fmgr_compress(self._parse_fops(fops))

    def fmgr_status(self,persistentId):
        return self.fmgr.status(persistentId)

    def ops_execute(self,fops,bucket,key):
        f = int(self.cfg.force)
        if self.cfg.separate:
            separate = int(self.cfg.separate)
        else:
            separate = 0
        notifyurl = self.cfg.notifyurl or ''
        return self.pfops.execute(fops,bucket,key,f,separate,notifyurl)
 
    def ops_status(self,persistentId):
        return self.pfops.fops_status(persistentId)

    def wslive_list(self,channelname, startTime, endTime, bucket, start=None, limit=None):
        return self.wsl.wslive_list( channelname, startTime, endTime, bucket, start, limit)




#!/usr/bin/python
## -*- coding: utf-8 -*-

from wcs.services.mgrbase import MgrBase
from wcs.commons.http import _post
from wcs.commons.http import _get
from wcs.commons.util import https_check

from wcs.commons.logme import debug, error

class Fmgr(MgrBase):
    """高级资源管理类
    高级资源管理继承自mgrbase
    """

    def __init__(self, auth,url):
        super(Fmgr, self).__init__(auth,url)

    def _fmgr_commons(self, reqdata, method):
        url = https_check('{0}/fmgr/{1}'.format(self.mgr_host,method))
        debug('Request body is: %s' % (reqdata))
        debug('Start to execute opration: %s' % method)
        return _post(url=url, data=reqdata, headers=super(Fmgr, self)._gernerate_headers(url, body=reqdata)) 
        
    def fmgr_move(self, reqdata):
        return self._fmgr_commons(reqdata, 'move')

    def fmgr_copy(self, reqdata):
        return self._fmgr_commons(reqdata, 'copy')
   
    def fmgr_fetch(self, reqdata):
        return self._fmgr_commons(reqdata, 'fetch') 

    def fmgr_delete(self, reqdata):
        return self._fmgr_commons(reqdata,'delete')

    def prefix_delete(self, reqdata):
        return self._fmgr_commons(reqdata,'deletePrefix')

    def m3u8_delete(self, reqdata):
        return self._fmgr_commons(reqdata, 'deletem3u8')

    def status(self, persistentId):
        url = '{0}/fmgr/status?persistentId={1}'.format(self.mgr_host, persistentId)
        url = https_check(url)
        debug('Start to get status of persistentId: %s' % persistentId)
        return _get(url=url)    

    # 多文件压缩(https://wcs.chinanetcenter.com/document/API/Fmgr/compress)
    # add by laihy 20200224
    def fmgr_compress(self, reqdata):
        return self._fmgr_commons(reqdata, 'compress')


#!/usr/bin/python
## -*- coding: utf-8 -*-

import ast
import os
from os.path import expanduser
import sys
try:
    import simplejson as json
except (ImportError, SyntaxError):
    # simplejson does not support Python 3.2, it thows a SyntaxError
    # because of u'...' Unicode literals.
    import json  # noqa
import platform
import requests
from wcs.commons.config import Config
from requests.adapters import HTTPAdapter
import yaml
#config_file = os.path.join(expanduser("~"), ".wcscfg")
from wcs import __version__
_session = None

def __return_wrapper(resp):
    try:
        if resp.text != '':
            if 'X-Reqid' in resp.headers.keys():
                resp_header = {'x-reqid': resp.headers['X-Reqid']}
            else:
                resp_header = {'x-reqid': 'None'}
            try:
                return resp.status_code, json.loads(str(resp.text)), resp_header
            except Exception as e:
                response_body = yaml.load(resp.text)
                if 'code' in response_body.keys():
                    return response_body["code"],response_body,resp_header
                else:
                    return -2, {'message':resp.text}, resp_header
        else:
            if resp.status_code == 303:
                resp_header = {'x-reqid': resp.headers['X-Reqid']}
                return resp.status_code, resp.headers['Location'], resp_header
            resp_header = resp.headers
            return -2, {'message':'Message Body is None. Please check your URL.'}, resp_header
    except Exception as error:
        return -2, {'message':'Some abnormal error occurred on the server.'}, {'x-reqid': 'None'}


def _init():
    session = requests.Session()
    session.mount('http://', HTTPAdapter(max_retries=int(Config.connection_retries)))
    session.mount('https://', HTTPAdapter(max_retries=int(Config.connection_retries)))
    global _session
    _session = session


def _post(url, headers, data=None, files=None):
    #cfg = Config(config_file)
    cfg = Config()
    timeout = float(cfg.connection_timeout)
    if _session is None:
        _init()
    try:
        headers['user-agent'] = 'WCS-Python3-SDK-{0}(http://wcs.chinanetcenter.com)'.format(__version__)
        #headers['Expect'] = '100-conitnue'
        if cfg.keepalive == True:
            pass
        else:
            _session.keep_alive = False
            headers['Connection'] = 'close'
        if cfg.isverify:
            if cfg.returnUrl:
                r = _session.post(url=url, data=data, files=files, headers=headers, timeout=timeout, verify=True, allow_redirects=False)
            else:
                r = _session.post(url=url, data=data, files=files, headers=headers, timeout=timeout, verify=True)
        else:
            if cfg.returnUrl:
                r = _session.post(url=url, data=data, files=files, headers=headers, timeout=timeout, verify=False, allow_redirects=False)
            else:
                r = _session.post(url=url, data=data, files=files, headers=headers, timeout=timeout, verify=False)
    except Exception as e:
        errormessage = {'message':str(e)}
        return -1,errormessage,'None'
    return __return_wrapper(r)

def _get(url, headers=None):
    #cfg = Config(config_file)
    cfg = Config()
    timeout = float(cfg.connection_timeout)
    try:
        headers = headers or {}
        headers['user-agent'] = 'WCS-Python3-SDK-{0}(http://wcs.chinanetcenter.com)'.format(__version__)
        if cfg.isverify:
            r = requests.get(url, headers=headers, timeout=timeout, verify=True)
        else:
            r = requests.get(url, headers=headers, timeout=timeout, verify=False)
    except Exception as e:
        resp_header = {'x-reqid': 'None'}
        return -1,e,resp_header
    return __return_wrapper(r)



#!/usr/bin/python
## -*- coding: utf-8 -*-

import os,sys
import time
from os.path import expanduser
from multiprocessing import Lock
from multiprocessing.dummy import Pool as ThreadPool
from wcs.commons.config import Config
from wcs.commons.http import _post
from wcs.commons.logme import debug,warning,error
from wcs.commons.util import readfile,GetUuid
from wcs.commons.util import urlsafe_base64_encode,https_check
from wcs.commons.error_deal import  WcsSeriveError
from wcs.services.uploadprogressrecorder import UploadProgressRecorder

#config_file = os.path.join(expanduser("~"), ".wcscfg")

lock = Lock()

class MultipartUpload(object):
    """分片上传类
    分片上传引入了块和片，一个文件由一到多个块组成，而每个块由一到多个片组成，块大小和片大小是可配的。块之间可以并发上传也可以顺序上传，块内的片之间只能顺序上传
    分片上传通过导入UploadProgressRecorder实现断点续传
    Attributes:
        url: 上传域名
        token: 鉴权token
        path: 源文件路径
        size: 源文件大小
        key: 源文件名
        blocknum: 源文件块数目
        results: 读取上传进度到这个变量
        uploadBatch: upload id
        progress: 当前上传进度
        recorder: UploadProgressRecorder的实例
        cfg: Config的实例
        concurrency: 块并发度
        block_size: 块大小
        bput_size: 片大小
        
    """   
    def __init__(self,url):
        self.url = url
        self.token = ''
        self.path = ''
        self.size = 0
        self.key = ''
        self.blocknum = 0
        self.results = []
        self.uploadBatch = ''
        self.progress = 0
        self.recorder = None
        #self.cfg = Config(config_file)
        self.cfg = Config()
        try:
            self.concurrency = int(self.cfg.concurrency)
            self.block_size = int(self.cfg.block_size)
            self.bput_size = int(self.cfg.bput_size)
        except ValueError as e:
            error(u"Invalid value,please check .wcscfg file")
            sys.exit()
            

    def __need_retry(self,code):
        if code == 500 or code == -1:
            return True
        return False

    def _define_config(self,path,token,upload_id=None):
        self.token = token
        self.path = path
        self.size = os.path.getsize(self.path)
        self.key = os.path.basename(path)
        if self.size % self.block_size != 0:
            self.blocknum = int(self.size/self.block_size + 1)
        else:
            self.blocknum = int(self.size/self.block_size)
        self.results = []
        self.uploadBatch = upload_id or ''
        self.progress = 0
        self.modify_time = time.time() 

    def _record_upload_progress(self, result, size):
        result_dict = dict(zip(['offset', 'code', 'ctx'], result))
        result_dict['size'] = size
        if result_dict['code'] == 200:
            lock.acquire()
            self.progress += size
            debug('Current block size: %d, total upload size: %d' % (int(size), self.progress))
            lock.release()
        self.recorder.set_upload_record(result_dict['offset'],result_dict)

    def _records_parse(self, upload_id):
        records = self.recorder.get_upload_record()
        offsetlist = [i * (self.block_size) for i in range(0,self.blocknum)]
        debug(records)
        if records:
            self.uploadBatch = records['uploadBatch']
            self.results = records['upload_record']
            for record in self.results:
                try:
                    record = eval(record)
                except SyntaxError as e:
                    debug('Get ctx/offset fail,error ctx/offset:{0}'.format(record))

                except Exception as exc_e:
                    debug('Get ctx/offset fail,errorinfo:{0}'.format(exc_e))

                if record['code'] == 200:
                    offsetlist.remove(record['offset'])
                    blockid = record['offset']/self.block_size
                    if blockid < self.blocknum - 1:
                        self.progress += self.block_size
                    else:              
                        self.progress += self.size - (blockid * self.block_size)       
        return offsetlist
    
    def _make_bput_post(self, ctx, bputnum, bput_next):
        url = https_check(self.__bput_url(ctx, bputnum*self.bput_size))
        headers = self.__generate_headers()
        return _post(url=url, headers=headers, data=bput_next)

    def __bput_url(self, ctx, offset):
        return '{0}/bput/{1}/{2}'.format(self.url, ctx, offset)

    def __block_url(self,size,blocknum):
        return '{0}/mkblk/{1}/{2}'.format(self.url, size, int(blocknum))

    def __file_url(self):
        url = ['{0}/mkfile/{1}'.format(self.url, self.size)]
        #if self.params:
        #    for k, v in self.params.items():
        #        url.append('x:{0}/{1}'.format(k, urlsafe_base64_encode(v)))
        url = '/'.join(url)
        return url

    def __generate_headers(self):
        headers = {'Authorization':self.token}
        headers['uploadBatch'] = self.uploadBatch
        return headers

    def _mlk_url(self, offset):
        blockid = offset/self.block_size
        if blockid < self.blocknum - 1:
            size = self.block_size
        else:
            size = int(self.size - (blockid * self.block_size))
        return self.__block_url(int(size), blockid), size

    def _make_block(self, offset):
        url,size = self._mlk_url(offset)
        url = https_check(url)
        headers = self.__generate_headers()        
        try:
            mkblk_retries = int(self.cfg.mkblk_retries)
        except ValueError as e:
            warning('parameter mkblk_retries is invalid, so use default value 3')
            mkblk_retries = 3
        with open(self.path, 'rb') as f:
            bput = readfile(f, offset, self.bput_size)
            blkcode, blktext,_ = _post(url=url,headers=headers, data=bput)
            while mkblk_retries and self.__need_retry(blkcode):
                debug('make block fail.retry upload')
                blkcode, blktext,_ = _post(url=url, headers=headers, data=bput)
                mkblk_retries -= 1
            if blkcode != 200:
                result = [offset, blkcode, blktext['message']]
                debug('make block fail,code :{0},message :{1}'.format(blkcode, blktext))
            else:
                result = self._make_bput(f, blktext['ctx'], offset)
        self._record_upload_progress(result,size)
        return blkcode
    
    def _make_bput(self, f, ctx, offset):
        bputnum = 1
        offset_next = offset + self.bput_size
        bput_next = readfile(f, offset_next, self.bput_size)
        bputcode = 200
        bputtext = {'ctx' : ctx}
        try:
            bput_retries = int(self.cfg.bput_retries)
        except ValueError as e:
            warning('parameter bput_retries is invalid, so use default value 3')
            bput_retries = 3
        while bput_next and bputnum < self.block_size/self.bput_size:
            bputcode, bputtext, _ = self._make_bput_post(ctx, bputnum, bput_next)
            while bput_retries and self.__need_retry(bputcode):
                debug('bput fail.retry upload')
                bputcode, bputtext, _ = self._make_bput_post(ctx, bputnum, bput_next)
                bput_retries -= 1
            if bputcode != 200:
                return offset, bputcode, bputtext['message']
            ctx = bputtext['ctx']
            offset_next = offset + bputtext['offset']
            bput_next = readfile(f, offset_next, self.bput_size)
            bputnum += 1
        return offset, bputcode, bputtext['ctx']
 
    def _is_complete(self):
        self.results = self.recorder.get_upload_record()
        debug(self.results)
        if len(self.results['upload_record']) < self.blocknum:
            return 0
        for result in self.results['upload_record']:
            result = eval(result)
            if result['code'] != 200:
                return 0
        return 1

    def _get_failoffsets(self):
        offsetlist = [i *  int(self.block_size) for i in range(0, self.blocknum)]
        if self.results:
            for result in self.results['upload_record']:
                result = eval(result)
                if result['offset'] in offsetlist:
                    offsetlist.remove(result['offset'])
        return offsetlist
        
    def _get_blkstatus(self):
        blkstatus = []
        for offset in [i * (self.block_size) for i in range(0,self.blocknum)]:
            for result in self.results['upload_record']:
                result = eval(result)
                if offset == result['offset']:
                    blkstatus.append(result['ctx'])
        return blkstatus

    def _make_file(self):
        try:
            mkfile_retries = int(self.cfg.mkfile_retries)
        except ValueError as e:
            warning(u"parameter mkfile_retries is invalid, so use default value 3")
            mkfile_retries = 3
        blkstatus = self._get_blkstatus()
        url = https_check(self.__file_url())
        body = ','.join(blkstatus)
        headers = self.__generate_headers()
        code,text,logid = _post(url=url,headers=headers,data=body)
        while mkfile_retries and self.__need_retry(code):
            debug('make file fail.retry upload')
            code,text,logid = _post(url=url,headers=headers,data=body)
        self.recorder.delete_upload_record()
        return code,text,logid

    def _initial_records(self):
        self.uploadBatch = GetUuid()
        debug('New upload id: %s' % self.uploadBatch)
        self.recorder = UploadProgressRecorder(self.uploadBatch)

    def upload(self,path,token,upload_id=None):
        self._define_config(path,token,upload_id)
        if upload_id:
            self.recorder = UploadProgressRecorder(upload_id)
            offsets = self._records_parse(upload_id)
            debug('Uncomplete offsetlist: %s, uploadid: %s, complete size: %d' % (offsets, upload_id, self.progress))
        else:
            debug("Now start a new multipart upload task")
            self._initial_records()
            offsets = [i * (self.block_size) for i in range(0,self.blocknum)]

        if len(offsets) != 0:
            debug('There are %d offsets need to upload' % (len(offsets)))
            debug('Now start upload file blocks') 
            if self.concurrency > 0:
                pool = ThreadPool(self.concurrency)
                pool.map(self._make_block, offsets)
                pool.close()
                pool.join()

            elif self.concurrency == 0:
                for offset in offsets:
                    return_code = self._make_block(offset)
                    if 400 <= int(return_code) <= 499:
                        debug('Single-Thread,attempt authentication failed,exit the task.')
                        sys.exit()
            else:
                raise ValueError('Invalid concurrency')
                sys.exit()

        if self._is_complete():
            debug('Now all blocks have upload suc.')
            return self._make_file()
        else:
            fail_list = self._get_failoffsets() 
            upload_record = str(Config.tmp_record_folder) + self.uploadBatch
            debug('Sorry! Mulitpart upload fail,more detail see %s' % upload_record)
            return self.results
            #raise Exception("Multipart upload fail")

    def smart_upload(self,path,token,upload_id=None):
        self._define_config(path,token,upload_id)
        if upload_id:
            self.recorder = UploadProgressRecorder(upload_id)
            offsets = self._records_parse(upload_id)
            debug('Uncomplete offsetlist: %s, uploadid: %s, complete size: %d' % (offsets, upload_id, self.progress))

        else:
            debug("Now start a new multipart upload task")
            self._initial_records()
            offsets = [i * (self.block_size) for i in range(0,self.blocknum)]

        if len(offsets) != 0:
            debug('There are %d offsets need to upload' % (len(offsets)))
            debug('Now start upload file blocks')
            if self.concurrency > 0:
                pool = ThreadPool(self.concurrency)
                pool.map(self._make_block, offsets)
                pool.close()
                pool.join()

            elif self.concurrency == 0:
                for offset in offsets:
                    return_code = self._make_block(offset)
                    if 400 <= int(return_code) <= 499:
                        debug('Single-Thread,attempt authentication failed,exit the task.')
                        sys.exit()
            else:
                raise ValueError('Invalid concurrency')
                sys.exit()

        if self._is_complete():
            debug('Now all blocks have upload suc.')
            mkfile_result = self._make_file()
            if 200 <= int(mkfile_result[0]) < 400:
                return mkfile_result
            else:
                raise WcsSeriveError("Make file fail,please upload file again")
        else:
            fail_list = self._get_failoffsets()
            upload_record = str(Config.tmp_record_folder) + self.uploadBatch
            debug('Sorry! Mulitpart upload fail,more detail see %s' % upload_record)
            # return self.results
            raise WcsSeriveError("Multipart upload fail,please upload file again")


__version__ = '5.0.17'

from .commons.auth import Auth
from .commons.compat import urlparse 
from .commons.http import _post,_get
from .commons.util import readfile,file_to_stream,etag

from .services.client import Client
from .services.filemanager import BucketManager
from .services.mgrbase import MgrBase
from .services.persistentfop import PersistentFop
from .services.simpleupload import SimpleUpload
from .services.multipartupload import MultipartUpload
from .services.uploadprogressrecorder import UploadProgressRecorder
from .services.fmgr import Fmgr
from .services.wslive import WsLive



#!/usr/bin/python
## -*- coding: utf-8 -*-

from wcs.services.mgrbase import MgrBase
from wcs.commons.http import _post
from wcs.commons.http import _get
from wcs.commons.util import urlsafe_base64_encode, https_check, entry
from wcs.commons.config import Config
from wcs.commons.logme import debug, error
class BucketManager(MgrBase):
    """资源管理类
    资源管理类继承自mgrbase
    """
    def __init__(self, auth,mgr_url):
        super(BucketManager, self).__init__(auth,mgr_url)

    def _limit_check(self):
        n = 0
        while n < 1001:
            yield n
            n += 1

    def _make_delete_url(self, bucket, key):
        return '{0}/delete/{1}'.format(self.mgr_host, entry(bucket, key))

    def delete(self, bucket, key):
        url = https_check(self._make_delete_url(bucket, key))
        debug('Start to post request of delete %s:%s' % (bucket, key))
        return _post(url=url, headers=super(BucketManager, self)._gernerate_headers(url))

    def _make_filestat_url(self, bucket, key):
        return '{0}/stat/{1}'.format(self.mgr_host, entry(bucket, key))

    def stat(self, bucket, key):
        url = https_check(self._make_filestat_url(bucket, key))
        debug('Start to get the stat of %s:%s' % ( bucket, key))
        return _get(url=url, headers=super(BucketManager, self)._gernerate_headers(url))
 
    def _make_url(self, operation, param):
        url = ['{0}/{1}'.format(self.mgr_host,operation)]
        if param:
            url.append(super(BucketManager, self)._params_parse(param))
        url = '?'.join(url)
        return url
    
    def bucketlist(self, bucket, prefix=None, marker=None, limit=None, mode=None,starttime=None,endtime=None):
        options = {
            'bucket': bucket,
        }
        if marker:
            options['marker'] = marker
        if limit:
            if limit in self._limit_check():
                options['limit'] = limit
            else:
                error('Invalid limit ! Please redefine limit')
                raise ValueError("Invalid limit")
        if prefix:
            options['prefix'] = urlsafe_base64_encode(prefix)
        if mode == 1 or mode == 0:
            options['mode'] = mode

        if starttime:
            options['startTime'] = starttime
        if endtime:
            options['endTime'] = endtime

        url = https_check(self._make_url('list', options))
        if options:
            debug('List options is %s' % options)
        debug('List bucket %s' % bucket)
        return _get(url=url, headers=super(BucketManager, self)._gernerate_headers(url))

    def _make_move_url(self, srcbucket, srckey, dstbucket, dstkey):
        src = urlsafe_base64_encode('%s:%s' % (srcbucket, srckey))
        dst = urlsafe_base64_encode('%s:%s' % (dstbucket, dstkey)) 
        url = '{0}/move/{1}/{2}'.format(self.mgr_host, src, dst)
        return https_check(url)

    def move(self, srcbucket, srckey, dstbucket, dstkey):
        url = self._make_move_url(srcbucket, srckey, dstbucket, dstkey)
        debug('Move object %s from %s to %s:%s' % (srckey, srcbucket, dstbucket, dstkey))
        return _post(url=url, headers=super(BucketManager, self)._gernerate_headers(url))
 
    def _make_copy_url(self, srcbucket, srckey, dstbucket, dstkey):
        src = urlsafe_base64_encode('%s:%s' % (srcbucket, srckey))
        dst = urlsafe_base64_encode('%s:%s' % (dstbucket, dstkey))
        url = '{0}/copy/{1}/{2}'.format(self.mgr_host, src, dst)
        return https_check(url)

    def copy(self, srcbucket, srckey, dstbucket, dstkey):
        url = self._make_copy_url(srcbucket, srckey, dstbucket, dstkey)
        debug('Copy object %s from %s to %s:%s' % (srckey, srcbucket, dstbucket, dstkey))
        return _post(url=url, headers=super(BucketManager, self)._gernerate_headers(url))

    def setdeadline(self, bucket, key, deadline):
        url = https_check('{0}/setdeadline'.format(self.mgr_host))
        param = { 'bucket' : urlsafe_base64_encode(bucket)}
        param['key'] = urlsafe_base64_encode(key) 
        param['deadline'] = deadline 
        body = super(BucketManager, self)._params_parse(param)
        debug('Set deadline of %s to %s' % (key, deadline))
        return _post(url=url, data=body, headers=super(BucketManager, self)._gernerate_headers(url, body))

    def bucket_list(self):
        url = '{0}/bucket/list'.format(self.mgr_host)
        url = https_check(url)
        debug('Now start to list buckets')
        return _get(url=url, headers=super(BucketManager, self)._gernerate_headers(url))

    def bucket_stat(self, name, startdate, enddate):
        encode_name = urlsafe_base64_encode(name)
        options = {'name':encode_name, 'startdate':startdate, 'enddate':enddate}
        url = https_check(self._make_url('bucket/stat', options))
        debug('Now check storage of %s from %s to %s' % (name, startdate, enddate))
        return _get(url=url, headers=super(BucketManager, self)._gernerate_headers(url))

    def bucket_statistics(self, name, stype, startdate, enddate, isListDetails='false'):
        encode_name = urlsafe_base64_encode(name)
        options = {'name':encode_name, 'type':stype, 'startdate':startdate, 'enddate':enddate}
        url = https_check(self._make_url('bucket/statistics', options))
        debug('Now get bucket %s of %s from %s to %s' % (stype, name, startdate, enddate))
        return _get(url=url, headers=super(BucketManager, self)._gernerate_headers(url))


    # 图片鉴定（https://wcs.chinanetcenter.com/document/API/Image-op/imageDetect）
    # add by laihy 20200609
    def image_detect(self, image, dtype, bucket):
        url = https_check('{0}/imageDetect'.format(self.mgr_host))
        param = { 'image' : urlsafe_base64_encode(image)}
        param['type'] = dtype
        param['bucket'] = bucket 
        body = super(BucketManager, self)._params_parse(param)
        debug('image detect for %s to %s' % (image, dtype))
        return _post(url=url, data=body, headers=super(BucketManager, self)._gernerate_headers(url, body))


# -*- coding: utf-8 -*-

class PutPolicy(object):
    def __init__(self):

        self.putpolicy = {}
        self.policy = set(
                ['scope', 'deadline', 'saveKey', 'fsizeLimit', 'overwrite', 'returnUrl', 'returnBody', 'callbackUrl',
                 'callbackBody', 'persistentNotifyUrl', 'persistentOps', 'separate', 'instant', 'detectNotifyRule',
                 'detectNotifyURL','hashAlgorithm'])

    def set_conf(self, key, value):

        if key in self.policy:
            self.putpolicy[key] = value
        else:
            print("invalid putpolicy param\n")

    def get_conf(self, key):

        if key in self.policy:
            return key + ":" + self.putpolicy[key]
        else:
            print("invalid putpolicy param\n")

    def dump_policy(self, cfg):
        if cfg.overwrite:
            self.set_conf('overwrite', int(cfg.overwrite))
        if cfg.returnUrl:
            self.set_conf('returnUrl', str(cfg.returnUrl))
        if cfg.returnBody:
            self.set_conf('returnBody', str(cfg.returnBody))
        if cfg.callbackUrl:
            self.set_conf('callbackUrl', str(cfg.callbackUrl))
        if cfg.callbackBody:
            self.set_conf('callbackBody', str(cfg.callbackBody))
        if cfg.persistentNotifyUrl:
            self.set_conf('persistentNotifyUrl', str(cfg.persistentNotifyUrl))
        if cfg.persistentOps:
            self.set_conf('persistentOps', str(cfg.persistentOps))
        if cfg.contentDetect:
            self.set_conf('contentDetect', str(cfg.contentDetect))
        if cfg.detectNotifyURL:
            self.set_conf('detectNotifyURL', str(cfg.detectNotifyURL))
        if cfg.detectNotifyRule:
            self.set_conf('detectNotifyRule', str(cfg.detectNotifyRule))
        if cfg.hashAlgorithm:
            self.set_conf('hashAlgorithm', str(cfg.hashAlgorithm))



#!/usr/bin/python
## -*- coding: utf-8 -*-
from wcs.services.mgrbase import MgrBase
from wcs.commons.config import Config
from wcs.commons.http import _get
from wcs.commons.logme import debug,error 
from wcs.commons.util import https_check

MGR = Config.mgr_url

class WsLive(MgrBase):
    """直播录制文件查询类
    该类继承自mgrbase
    """
    def __init__(self, auth,url):
        super(WsLive, self).__init__(auth,url)

    def _make_list_url(self, param):
        url = ['{0}/wslive/list'.format(self.mgr_host)] 
        if param:
            url.append(super(WsLive, self)._params_parse(param))
        url = '?'.join(url)
        return url

    def wslive_list(self, channelname, startTime, endTime, bucket, start=None, limit=None):
        query = {
            'channelname' : channelname,
            'startTime' : startTime,
            'endTime' : endTime,
            'bucket' : bucket,
        }
        if start is not None:
            query['start'] = start
        if limit is not None:
            query['limit'] = limit
        url = https_check(self._make_list_url(query))
        if query is not None:
            debug('List params is %s' % query)
        debug('List bucket %s' % bucket)
        return _get(url=url, headers=super(WsLive, self)._gernerate_headers(url)) 



# -*- coding: utf-8 -*-
from hashlib import sha1
from base64 import urlsafe_b64encode, urlsafe_b64decode
import base64
import os,re
import logging
import crcmod

from wcs.commons.compat import b, s
import tempfile
import random
import string
from wcs.commons.config import Config
from wcs.commons.error_deal import ParameterError
try:
    import zlib
    binascii = zlib
except ImportError:
    zlib = None
    import binascii


def urlsafe_base64_encode(data):
    """urlsafe的base64编码:
    对提供的数据进行urlsafe的base64编码。规格参考：
    Args:
        data: 待编码的数据，一般为字符串
    Returns:
        编码后的字符串
    """
    ret = urlsafe_b64encode(b(data))
    return s(ret)

def urlsafe_base64_decode(data):
    """urlsafe的base64解码:
    对提供的urlsafe的base64编码的数据进行解码
    Args:
        data: 待解码的数据，一般为字符串
    Returns:
        解码后的字符串。
    """
    ret = urlsafe_b64decode(s(data))
    return ret

def file_crc32(filePath):
    """计算文件的crc32检验码:
    Args:
        filePath: 待计算校验码的文件路径
    Returns:
        文件内容的crc32校验码。
    """
    crc = 0
    with open(filePath, 'rb') as f:
        for block in _file_iter(f,0,Config.block_size):
            crc = binascii.crc32(block, crc) & 0xFFFFFFFF
    return crc

def crc32(data):
    """计算输入流的crc32检验码:
    Args:
        data: 待计算校验码的字符流
    Returns:
        输入流的crc32校验码。
    """
    return binascii.crc32(b(data)) & 0xffffffff

def crc64(data):
    """计算输入流的crc64检验码:
    Args:
        data: 待计算校验码的字符流
    Returns:
        输入流的crc64校验码。
    """
    try:
        _POLY = 0x142F0E1EBA9EA3693
        _XOROUT = 0xffffffffffffffff
        c64 = crcmod.mkCrcFun(_POLY, initCrc=0, xorOut=_XOROUT, rev=True)
        return str(c64(data))
    except Exception as error:
        raise ParameterError('calculation failed. {0}'.format(error))

def file_crc64(file,block_size=16*1024,is_path=True):
    """计算整个文件的crc64检验码:
    Args:
        filePath: 待计算校验码的文件路径
        block_size:每次遍历计算的文件大小，默认16K
        is_path: 待计算的文件时流还是文件路径，默认文件路径
    Returns:
        文件内容的crc64校验码。
    """
    _POLY = 0x142F0E1EBA9EA3693
    _XOROUT = 0xffffffffffffffff
    _initCrc = 0
    local_crc64 = 0
    c64 = crcmod.mkCrcFun(_POLY, initCrc=_initCrc, xorOut=_XOROUT, rev=True)
    try:
        if is_path:
            with open(file, 'rb') as f:
                while True:
                    data = f.read(int(block_size))
                    if not data:
                        break
                    crc64 = c64(data,local_crc64)
                    local_crc64 = crc64
        else:
            start_size = 0
            while True:
                data = file[start_size:start_size+int(block_size)]
                start_size += block_size
                if not data:
                    break
                crc64 = c64(data,local_crc64)
                local_crc64 = crc64
        return str(local_crc64)
    except IOError:
        raise ParameterError('file does not exist')
    except Exception as error:
        raise ParameterError('calculation failed. {0}'.format(error))

def _file_iter(input_stream,offset,size):
    """读取输入流:
    Args:
        input_stream: 待读取文件的二进制流
        size:         二进制流的大小
    Raises:
        IOError: 文件流读取失败
    """
    input_stream.seek(int(offset))
    d = input_stream.read(int(size))
    while d :
        yield d
        d = input_stream.read(int(size))

def readfile(input,offset,size):
    input.seek(offset)
    d = input.read(size)
    if d:
        return d

def file_to_stream(path):
    return open(path, "rb")

def _sha1(data):
    """单块计算hash:
    Args:
        data: 待计算hash的数据
    Returns:
        输入数据计算的hash值
    """
    h = sha1()
    h.update(data)
    return h.digest()


def etag_stream(input_stream):
    """计算输入流的etag:
    Args:
        input_stream: 待计算etag的二进制流
    Returns:
        输入流的etag值
    """
    array = [_sha1(block) for block in _file_iter(input_stream, 0, int(Config.block_size))]
    if len(array) == 0:
        array = [_sha1(b'')]
    if len(array) == 1:
        data = array[0]
        prefix = b'\x16'
    else:
        sha1_str = b('').join(array)
        data = _sha1(sha1_str)
        prefix = b'\x96'
    return urlsafe_base64_encode(prefix + data)


def etag(filePath):
    """计算文件的etag:
    Args:
        filePath: 待计算etag的文件路径
    Returns:
        输入文件的etag值
    """
    with open(filePath, 'rb') as f:
        return etag_stream(f)


def entry(bucket, key):
    """计算wcs API中的数据格式:
    Args:
        bucket: 待操作的空间名
        key:    待操作的文件名
    Returns:
        符合wcs API规格的数据格式
    """
    if key is None:
        return urlsafe_base64_encode('{0}'.format(bucket))
    else:
        return urlsafe_base64_encode('{0}:{1}'.format(bucket, key))

def GetUuid():
    chars = string.ascii_letters+string.digits
    return ''.join([random.choice(chars) for i in range(32)])

def https_check(url):
    # by caiyz 20180408
    reobj  = re.compile('^http://|https://')
    if reobj.match(url.lower()):
        return url
    else:
        if Config.ishttps:
            return "https://" + url
        else:
            return "http://" + url
    # if Config.ishttps:
    #     return "https://" + url
    # else:
    #     return "http://" + url



import io
import os
import re

try:
    import setuptools
    setup = setuptools.setup
except ImportError:
    setuptools = None
    from distutils.core import setup


def read(name):
    return open(os.path.join(os.path.dirname(__file__), name)).read()


packages = [
    'wcs',
    'wcs.services',
    'wcs.commons',
]


def find_version(*file_paths):
    version_file = read(*file_paths)
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError("Unable to find version string.")


setup(
        name='wcs-python3-sdk',
        version=find_version("wcs/__init__.py"),
        description='Wangsu Cloud Storage Command Tool',
        long_description='see:\nhttps://wcs.chinanetcenter.com/document/SDK\n',
        author='ChinaNetCenter.Wangsu Cloud Strage',
        maintainer='https://github.com/Wangsu-Cloud-Storage',
        author_email='cdn_team_storage_wcs@chinanetcenter.com',
        url='https://wcs.chinanetcenter.com/document/SDK',
        packages=packages,
        scripts=['wcscmd'],
        license='MIT',
        classifiers=[
            'Intended Audience :: Developers',
            'Operating System :: OS Independent',
            'Programming Language :: Python',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
            'Topic :: Software Development :: Libraries :: Python Modules'
        ],
        install_requires=['requests>=2.19.1', 'argparse', 'lockfile', 'requests_toolbelt', 'pyyaml','crcmod>=1.7'],
        entry_points={
            'console_scripts': [
                'wcs_etag_cal = wcs.commons.etag_files:main','wcs_crc64_cal = wcs.commons.crc64_files:main']
        }
)



#!/usr/bin/python
## -*- coding: utf-8 -*-
import requests
from io import BytesIO
from requests_toolbelt import MultipartEncoder

from wcs.services.simpleupload import SimpleUpload

class StreamUpload(SimpleUpload):
    """流地址上传类
    该类继承自普通上传类，用于流地址上传，与普通上传相比差别是源数据来源于一个公网URL由URL获取到数据内容之后，不是落盘而是写入内存，再调用普通上传API完成上传
    """   
    def __init__(self, url):
        super(StreamUpload, self).__init__(url)

    def _gernerate_content(self, stream):
        memory = BytesIO()
        file = requests.get(stream)
        memory.write(file.content)
        return memory

    def upload(self, stream,token,key):
        memory = self._gernerate_content(stream)
        url,encoder,headers = super(StreamUpload, self)._gernerate_tool(memory,token,key)
        return super(StreamUpload,self)._upload(url,encoder,headers,memory)
